@@ -564,7 +564,6 @@ get_level_indent (HTMLClueFlow *flow,
 		  gint level,
 		  HTMLPainter *painter)
 {
-	int line_offset = 0;
 	guint indent = 0;
 	gint i = 0;
 
@@ -598,6 +597,12 @@ get_level_indent (HTMLClueFlow *flow,
 	return indent;
 }
 
+static void
+set_painter (HTMLObject *o, HTMLPainter *painter)
+{
+	HTML_CLUEFLOW (o)->indent_width = -1;
+}
+
 static guint
 get_indent (HTMLClueFlow *flow,
 	    HTMLPainter *painter)
@@ -848,8 +853,7 @@ layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 	x = start_lmargin = *lmargin;
 	o->ascent += o->y - old_y;
 
-	while (cur && !(cur->flags & HTML_OBJECT_FLAG_ALIGNED) && (HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE
-								   || (x < *rmargin || first))) {
+	while (cur && !(cur->flags & HTML_OBJECT_FLAG_ALIGNED)) {
 		HTMLFitType fit;
 		HTMLVAlignType valign;
 
@@ -949,7 +953,7 @@ static gboolean
 calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	HTMLClueFlow *cf = HTML_CLUEFLOW (o);
-	gint oa, od, ow, oi, padding;
+	gint oa, od, ow, padding;
 	gboolean leaf_children_changed_size = FALSE;
 	gboolean changed, changed_size = FALSE;
 
@@ -1108,7 +1112,7 @@ draw_gt_line (HTMLObject *cur, HTMLPainter *p, gint offset, gint x, gint y)
 
 	/* FIXME: cache items and glyphs? */
 	html_painter_calc_text_size (p, HTML_BLOCK_CITE, 
-				     strlen (HTML_BLOCK_CITE), NULL, NULL, &line_offset,
+				     strlen (HTML_BLOCK_CITE), NULL, NULL, 0, &line_offset,
 				     GTK_HTML_FONT_STYLE_SIZE_3, NULL,
 				     &w, &a, &d);
 
@@ -1116,7 +1120,7 @@ draw_gt_line (HTMLObject *cur, HTMLPainter *p, gint offset, gint x, gint y)
 	while (cy + a <= cur->ascent) {
 		/* FIXME: cache items and glyphs? */
 		html_painter_draw_text (p, x, y + cur->y - cy,
-					HTML_BLOCK_CITE, 1, NULL, NULL, 0);
+					HTML_BLOCK_CITE, 1, NULL, NULL, 0, 0);
 		cy += a + d;
 	}
 
@@ -1124,7 +1128,7 @@ draw_gt_line (HTMLObject *cur, HTMLPainter *p, gint offset, gint x, gint y)
 	while (cy + d <= cur->descent) {
 		/* FIXME: cache items and glyphs? */
 		html_painter_draw_text (p, x, y + cur->y + cy,
-					HTML_BLOCK_CITE, 1, NULL, NULL, 0);
+					HTML_BLOCK_CITE, 1, NULL, NULL, 0, 0);
 		cy += a + d;
 	}
 }
@@ -1241,7 +1245,7 @@ draw_item (HTMLObject *self, HTMLPainter *painter, gint x, gint y, gint width, g
 
 			len   = strlen (number);
 			/* FIXME: cache items and glyphs? */
-			html_painter_calc_text_size (painter, number, len, NULL, NULL, &line_offset,
+			html_painter_calc_text_size (painter, number, len, NULL, NULL, 0, &line_offset,
 						     html_clueflow_get_default_font_style (flow), NULL, &width, &asc, &dsc);
 			width += html_painter_get_space_width (painter, html_clueflow_get_default_font_style (flow), NULL);
 			html_painter_set_font_style (painter, html_clueflow_get_default_font_style (flow));
@@ -1249,7 +1253,7 @@ draw_item (HTMLObject *self, HTMLPainter *painter, gint x, gint y, gint width, g
 			/* FIXME: cache items and glyphs? */
 			html_painter_draw_text (painter, self->x + first->x - width + tx,
 						self->y - self->ascent + first->y + ty,
-						number, strlen (number), NULL, NULL, 0);
+						number, strlen (number), NULL, NULL, 0, 0);
 		}
 		g_free (number);
 	}
@@ -1693,24 +1697,6 @@ save (HTMLObject *s,
 	return TRUE;
 }
 
-static gint
-string_append_nonbsp (GString *out, guchar *s, gint length)
-{
-	gint len = length;
-
-	while (len--) {
-		if (IS_UTF8_NBSP (s)) {
-			g_string_append_c (out, ' ');
-			s += 2;
-			len--;
-		} else {
-			g_string_append_c (out, *s);
-			s++;
-		}
-	}
-	return length;
-}
-
 static void
 write_item_marker (GString *pad_string, HTMLClueFlow *flow)
 {
@@ -1882,7 +1868,7 @@ save_plain (HTMLObject *self,
 				align_pad--;
 			}
 
-			s += string_append_nonbsp (out, s, len);
+			s += html_engine_save_string_append_nonbsp (out, s, len);
 			
 			/* Trim the space at the end */
 			while (*s == ' ' || IS_UTF8_NBSP (s)) 
@@ -2238,6 +2224,7 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 	object_class->relayout = relayout;
 	object_class->get_recursive_length = get_recursive_length;
 	object_class->get_clear = get_clear;
+	object_class->set_painter = set_painter;
 
 	klass->get_default_font_style = get_default_font_style;
 
@@ -2748,7 +2735,7 @@ static gchar *
 end_of_word (gchar *ct, gboolean cited)
 {
 	gunichar uc, ucn;
-	gchar *cn, *cnn;
+	gchar *cn;
 	gboolean cited2;
 
 	cited2 = FALSE;
@@ -2756,8 +2743,8 @@ end_of_word (gchar *ct, gboolean cited)
 	       && (uc = g_utf8_get_char (ct))
 	       && (cn = g_utf8_next_char (ct))
 	       && (html_selection_spell_word (uc, &cited2)
-		   || (!cited && cited2
-		       && *cn && (cnn = g_utf8_next_char (cn)) && (ucn = g_utf8_get_char (cnn)) && g_unichar_isalpha (ucn)))) {
+		   || (!cited && cited2)
+		   || (cited && cited2 && (ucn = g_utf8_get_char (cn)) && g_unichar_isalpha (ucn)))) {
 		ct = cn;
 		cited2 = FALSE;
 	}
@@ -2880,16 +2867,19 @@ html_clueflow_get_line_offset (HTMLClueFlow *flow, HTMLPainter *painter, HTMLObj
 	o = head = child;
 	while (o) {
 		o = head->prev;
-		if (o && o->y + o->descent - 1 < child->y - child->ascent)
-			break;
-		else
-			head = o;
+		if (o) {
+			if (o->y + o->descent - 1 < child->y - child->ascent)
+				break;
+			else
+				head = o;
+		}
 	}
 
 	if (HTML_IS_TEXT_SLAVE (head)) {
 		HTMLTextSlave *bol = HTML_TEXT_SLAVE (head);
-		line_offset = html_text_text_line_length (html_text_get_text (bol->owner, bol->posStart),
-							  0, bol->owner->text_len - bol->posStart);
+
+		html_text_text_line_length (html_text_get_text (bol->owner, bol->posStart),
+					    &line_offset, bol->owner->text_len - bol->posStart, NULL);
 		head = html_object_next_not_slave (head);
 	}
 
