@@ -29,6 +29,8 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtk.h>
 
 #include "gtkhtml.h"
 #include "gtkhtml-properties.h"
@@ -53,11 +55,11 @@
 struct _HTMLImageFactory {
 	HTMLEngine *engine;
 	GHashTable *loaded_images;
+	GdkPixbuf  *missing;
 };
 
 
 #define DEFAULT_SIZE 48
-#define ANIMATIONS(i) GTK_HTML_CLASS (GTK_WIDGET_GET_CLASS (i->image_ptr->factory->engine->widget))->properties->animations
 #define STRDUP_HELPER(i,j) if (i != j) {char *tmp = g_strdup (j); g_free(i); i = tmp;}
 
 HTMLImageClass html_image_class;
@@ -70,6 +72,8 @@ static gboolean            html_image_pointer_timeout           (HTMLImagePointe
 static gint                html_image_pointer_run_animation     (HTMLImagePointer *ip);
 static void                html_image_pointer_start_animation   (HTMLImagePointer *ip);
 
+static GdkPixbuf *         html_image_factory_get_missing       (HTMLImageFactory *factory);
+
 guint
 html_image_get_actual_width (HTMLImage *image, HTMLPainter *painter)
 {
@@ -314,7 +318,7 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 
 		style = html_clueflow_get_default_font_style (HTML_CLUEFLOW (o->parent));
 		/* FIXME: cache items and glyphs? */
-		html_painter_calc_text_size (painter, image->alt, g_utf8_strlen (image->alt, -1), NULL, NULL, &lo,
+		html_painter_calc_text_size (painter, image->alt, g_utf8_strlen (image->alt, -1), NULL, NULL, 0, &lo,
 					     style, NULL, &o->width, &o->ascent, &o->descent);
 	} else {
 		width = html_image_get_actual_width (image, painter);
@@ -341,7 +345,7 @@ draw_plain (HTMLObject *o, HTMLPainter *p, gint x, gint y, gint width, gint heig
 	if (img->alt && *img->alt) {
 		html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLTextColor)->color);
 		/* FIXME: cache items and glyphs? */
-		html_painter_draw_text (p, o->x + tx, o->y + ty - o->ascent, img->alt, g_utf8_strlen (img->alt, -1), NULL, NULL, 0);
+		html_painter_draw_text (p, o->x + tx, o->y + ty - o->ascent, img->alt, g_utf8_strlen (img->alt, -1), NULL, NULL, 0, 0);
 	}
 }
 
@@ -367,7 +371,7 @@ draw (HTMLObject *o,
 		return;
 
 	if (HTML_IS_PLAIN_PAINTER (painter)) {
-	draw_plain (o, painter, x, y, width, height, tx, ty);
+		draw_plain (o, painter, x, y, width, height, tx, ty);
 		return;
 	}
 
@@ -392,6 +396,9 @@ draw (HTMLObject *o,
 	} else
 		highlight_color = NULL;
 
+	base_x = o->x + tx + (image->border + image->hspace) * pixel_size;
+	base_y = o->y + ty + (image->border + image->vspace) * pixel_size - o->ascent;
+
 	if (pixbuf == NULL) {
 		gint vspace, hspace;
 
@@ -416,12 +423,23 @@ draw (HTMLObject *o,
 					 o->width - 2 * hspace,
 					 o->ascent + o->descent - 2 * vspace,
 					 GTK_HTML_ETCH_IN, 1);
+
+		if (ip->factory)
+			pixbuf = html_image_factory_get_missing (ip->factory);
+
+		if (pixbuf && 
+		    (o->width > gdk_pixbuf_get_width (pixbuf)) &&
+		    (o->ascent  + o->descent > gdk_pixbuf_get_height (pixbuf)))
+			html_painter_draw_pixmap (painter, pixbuf,
+						  base_x, base_y,
+						  gdk_pixbuf_get_width (pixbuf) * pixel_size,
+						  gdk_pixbuf_get_height (pixbuf) * pixel_size,
+						  highlight_color);
+			
+			
 		return;
 	}
 
-	base_x = o->x + tx + (image->border + image->hspace) * pixel_size;
-	base_y = o->y + ty + (image->border + image->vspace) * pixel_size - o->ascent;
-
 	scale_width = html_image_get_actual_width (image, painter);
 	scale_height = html_image_get_actual_height (image, painter);
 
@@ -1065,17 +1083,18 @@ html_image_pointer_queue_animation (HTMLImagePointer *ip)
 		ip->animation_timeout = g_timeout_add (delay, 
 						       (GtkFunction) html_image_pointer_run_animation, 
 						       (gpointer) ip);
-
+	else
+		ip->animation_timeout = 0;
 }
 
 static gint
 html_image_pointer_run_animation (HTMLImagePointer *ip)
 {
-	GdkPixbufAnimation      *ganim = ip->animation;
 	GdkPixbufAnimationIter  *iter = ip->iter;
 	HTMLEngine              *engine = ip->factory->engine;
-	gint nx, ny;
 	
+	g_return_val_if_fail (ip->factory != NULL, FALSE);
+
 	/* printf ("animation_timeout\n"); */
 	if (gdk_pixbuf_animation_iter_advance (iter, NULL)) {
 		GSList *cur;
@@ -1113,6 +1132,16 @@ html_image_pointer_stop_animation (HTMLImagePointer *ip)
 	}
 }
 
+static GdkPixbuf *
+html_image_factory_get_missing (HTMLImageFactory *factory)
+{
+	if (!factory->missing)
+		factory->missing = gtk_widget_render_icon (GTK_WIDGET (factory->engine->widget),
+							  GTK_STOCK_MISSING_IMAGE,
+							  GTK_ICON_SIZE_BUTTON, "GtkHTML.ImageMissing");
+	return factory->missing;
+}
+
 HTMLImageFactory *
 html_image_factory_new (HTMLEngine *e)
 {
@@ -1120,32 +1149,32 @@ html_image_factory_new (HTMLEngine *e)
 	retval = g_new (HTMLImageFactory, 1);
 	retval->engine = e;
 	retval->loaded_images = g_hash_table_new (g_str_hash, g_str_equal);
+	retval->missing = NULL;
 
 	return retval;
 }
 
 static gboolean
-cleanup_images (gpointer key, gpointer value, gpointer user_data)
+cleanup_images (gpointer key, gpointer value, gpointer free_everything)
 {
-	HTMLImagePointer *ptr;
-	gboolean retval = FALSE;
+	HTMLImagePointer *ip = value;
 
-	ptr = value;
-	/* user data means: NULL only clean, non-NULL free */
-	if (user_data){
-		if (ptr->interests != NULL) {
-			g_slist_free (ptr->interests);
-			ptr->interests = NULL;
+	/* free_everything means: NULL only clean, non-NULL free */
+	if (free_everything){
+		if (ip->interests != NULL) {
+			g_slist_free (ip->interests);
+			ip->interests = NULL;
 		}
 	}
 
 	/* clean only if this image is not used anymore */
-	if (!ptr->interests){
-		retval = TRUE;
-		html_image_pointer_unref (ptr);
+	if (!ip->interests){
+		html_image_pointer_unref (ip);
+		ip->factory = NULL;
+		return TRUE;
 	}
 
-	return retval;
+	return FALSE;
 }
 
 void
@@ -1162,6 +1191,10 @@ html_image_factory_free (HTMLImageFactory *factory)
 
 	g_hash_table_foreach_remove (factory->loaded_images, cleanup_images, factory);
 	g_hash_table_destroy (factory->loaded_images);
+
+	if (factory->missing)
+		gdk_pixbuf_unref (factory->missing);
+
 	g_free (factory);
 }
 
@@ -1194,6 +1227,10 @@ html_image_pointer_timeout (HTMLImagePointer *ip)
 	GSList *list;
 	HTMLImage *image;
 
+	ip->stall_timeout = 0;
+
+	g_return_val_if_fail (ip->factory != NULL, FALSE);
+
 	ip->stall = TRUE;
 
 	list = ip->interests;
@@ -1212,7 +1249,6 @@ html_image_pointer_timeout (HTMLImagePointer *ip)
 			list = list->next;
 		}
 	}
-	//ip->stall_timeout = 0;
 	return FALSE;
 }
 
@@ -1337,10 +1373,20 @@ html_image_factory_unregister (HTMLImageFactory *factory, HTMLImagePointer *poin
 	pointer->interests = g_slist_remove (pointer->interests, i);
 
 	html_image_pointer_unref (pointer);
+
 	if (pointer->refcount == 1) {
 		g_assert (pointer->interests == NULL);
-		/* printf ("remove %s\n", pointer->url); */
-		g_hash_table_remove (factory->loaded_images, pointer->url);
+		/*
+		 * FIXME The factory can be NULL if the image was from an iframe 
+		 * that has been destroyed and the image is living in the cut buffer
+		 * this isn't particularly clean and should be refactored.
+		 *
+		 * We really need a way to let cut objects know they are living outside
+		 * the normal flow.
+		 */
+		if (factory) 
+			g_hash_table_remove (factory->loaded_images, pointer->url);
+		pointer->factory = NULL;
 		html_image_pointer_unref (pointer);
 	}
 }
@@ -1366,6 +1412,7 @@ move_image_pointers (gpointer key, gpointer value, gpointer data)
 	HTMLImagePointer *ip  = HTML_IMAGE_POINTER (value);
 
 	ip->factory = dst;
+	
 	g_hash_table_insert (dst->loaded_images, ip->url, ip);
 	g_signal_emit_by_name (ip->factory->engine, "url_requested", ip->url, html_image_pointer_load (ip));
 
