@@ -42,8 +42,8 @@ HTMLTextSlaveClass html_text_slave_class;
 static HTMLObjectClass *parent_class = NULL;
 
 static GList * get_items (HTMLTextSlave *slave, HTMLPainter *painter);
-static GList * get_glyphs (HTMLTextSlave *slave, HTMLPainter *painter, gint line_offset);
-static GList * get_glyphs_part (HTMLTextSlave *slave, HTMLPainter *painter, guint offset, guint len, gint line_offset, GList **items);
+static GList * get_glyphs (HTMLTextSlave *slave, HTMLPainter *painter);
+static GList * get_glyphs_part (HTMLTextSlave *slave, HTMLPainter *painter, guint offset, guint len, GList **items);
 
 char *
 html_text_slave_get_text (HTMLTextSlave *slave)
@@ -93,13 +93,44 @@ copy (HTMLObject *self,
 }
 
 static guint
-get_words_width (HTMLText *text, HTMLPainter *p, guint start_word, guint words)
+get_words_width (HTMLTextSlave *slave, HTMLPainter *p, guint words)
 {
-	return words > 0
-		? text->word_width [start_word + words - 1]
-		- (start_word ? text->word_width [start_word - 1]
-		   + html_painter_get_space_width (p, html_text_get_font_style (text), text->face) : 0)
-		: 0;
+	HTMLText *text = slave->owner;
+	gint width;
+
+	if (words <= 0)
+		return 0;
+
+	width =  text->word_width [slave->start_word + words - 1]
+		- (slave->start_word ? text->word_width [slave->start_word - 1]
+		   + html_painter_get_space_width (p, html_text_get_font_style (text), text->face) : 0);
+
+	if (html_clueflow_tabs (HTML_CLUEFLOW (HTML_OBJECT (slave)->parent), p)) {
+		gchar *space, *str = html_text_slave_get_text (slave);
+		gint tabs, len, line_offset = html_text_slave_get_line_offset (slave, 0, p);
+
+		space = str;
+		while (words && space && *space) {
+			space = strchr (space, ' ');
+			words --;
+		}
+
+		if (!space || !*space)
+			len = slave->posLen;
+		else
+			len = g_utf8_pointer_to_offset (str, space);
+		/* printf ("width %d --> ", width); */
+		width += html_painter_get_space_width (p, html_text_get_font_style (text), text->face)*(html_text_text_line_length (str, &line_offset, len, &tabs) - len - tabs);
+		/* printf ("%d\n", width); */
+	}
+
+	return width;
+}
+
+static inline gint
+html_text_slave_get_start_byte_offset (HTMLTextSlave *slave)
+{
+	return html_text_slave_get_text (slave) - slave->owner->text;
 }
 
 static guint
@@ -107,39 +138,45 @@ calc_width (HTMLTextSlave *slave, HTMLPainter *painter, gint *asc, gint *dsc)
 {
 	HTMLText *text = slave->owner;
 	HTMLObject *next, *prev;
+	gint line_offset, tabs = 0, width = 0;
+
+	line_offset = html_text_slave_get_line_offset (slave, 0, painter);
+	if (line_offset != -1)
+		width += (html_text_text_line_length (html_text_slave_get_text (slave), &line_offset, slave->posLen, &tabs) - slave->posLen)*
+			html_painter_get_space_width (painter, html_text_get_font_style (text), text->face);
 
 	html_text_request_word_width (text, painter);
 	if (slave->posStart == 0 && slave->posLen == text->text_len) {
 		*asc = HTML_OBJECT (text)->ascent;
 		*dsc = HTML_OBJECT (text)->descent;
 
-		return text->word_width [text->words - 1];
-	}
-
-	next = HTML_OBJECT (slave)->next;
-	prev = HTML_OBJECT (slave)->prev;
-	if ((prev && HTML_OBJECT_TYPE (prev) == HTML_TYPE_TEXTSLAVE
-	     && slave->start_word == HTML_TEXT_SLAVE (prev)->start_word)
-	    || (next && HTML_OBJECT_TYPE (next) == HTML_TYPE_TEXTSLAVE
-		&& slave->start_word == HTML_TEXT_SLAVE (next)->start_word)) {
-		gint line_offset = -1;
-		gint width;
-
-		html_painter_calc_text_size (painter, html_text_slave_get_text (slave), slave->posLen, get_items (slave, painter), get_glyphs (slave, painter, line_offset),
-					     &line_offset, html_text_get_font_style (text),
-					     text->face, &width, asc, dsc);	
-		return width;
+		width += text->word_width [text->words - 1];
 	} else {
-		gint width;
+		next = HTML_OBJECT (slave)->next;
+		prev = HTML_OBJECT (slave)->prev;
+		if ((prev && HTML_OBJECT_TYPE (prev) == HTML_TYPE_TEXTSLAVE
+		     && slave->start_word == HTML_TEXT_SLAVE (prev)->start_word)
+		    || (next && HTML_OBJECT_TYPE (next) == HTML_TYPE_TEXTSLAVE
+			&& slave->start_word == HTML_TEXT_SLAVE (next)->start_word)) {
+			gint line_offset = -1;
+			gint w;
 
-		width = get_words_width (text, painter, slave->start_word,
-					 (next && HTML_OBJECT_TYPE (next) == HTML_TYPE_TEXTSLAVE
-					  ? HTML_TEXT_SLAVE (next)->start_word : text->words) - slave->start_word);
-		*asc = HTML_OBJECT (text)->ascent;
-		*dsc = HTML_OBJECT (text)->descent;
-
-		return width;
+			html_painter_calc_text_size (painter, html_text_slave_get_text (slave), slave->posLen,
+						     get_items (slave, painter), get_glyphs (slave, painter),
+						     html_text_slave_get_start_byte_offset (slave),
+						     &line_offset, html_text_get_font_style (text),
+						     text->face, &w, asc, dsc);
+			width += w + tabs*html_painter_get_space_width (painter, html_text_get_font_style (text), text->face);
+		} else {
+			width += get_words_width (slave, painter,
+						  (next && HTML_OBJECT_TYPE (next) == HTML_TYPE_TEXTSLAVE
+						   ? HTML_TEXT_SLAVE (next)->start_word : text->words) - slave->start_word);
+			*asc = HTML_OBJECT (text)->ascent;
+			*dsc = HTML_OBJECT (text)->descent;
+		}
 	}
+
+	return width;
 }
 
 inline static void
@@ -164,8 +201,8 @@ get_offset_for_bounded_width (HTMLTextSlave *slave, HTMLPainter *painter, gint *
 	char *buffer = html_text_slave_get_text (slave);
 
 	len = (lower + upper) / 2;
-	html_painter_calc_text_size (painter, buffer, len, get_items (slave, painter), get_glyphs (slave, painter, line_offset), &line_offset,
-				     html_text_get_font_style (text), text->face, &width, &asc, &dsc);
+	html_painter_calc_text_size (painter, buffer, len, get_items (slave, painter), get_glyphs (slave, painter), html_text_slave_get_start_byte_offset (slave),
+				     &line_offset, html_text_get_font_style (text), text->face, &width, &asc, &dsc);
 	while (lower < upper) {
 		if (width > max_width)
 			upper = len - 1;
@@ -173,8 +210,8 @@ get_offset_for_bounded_width (HTMLTextSlave *slave, HTMLPainter *painter, gint *
 			lower = len + 1;
 		len = (lower + upper) / 2;
 		line_offset = -1;
-		html_painter_calc_text_size (painter, buffer, len, get_items (slave, painter), get_glyphs (slave, painter, line_offset), &line_offset, 
-					     html_text_get_font_style (text), text->face, &width, &asc, &dsc);
+		html_painter_calc_text_size (painter, buffer, len, get_items (slave, painter), get_glyphs (slave, painter), html_text_slave_get_start_byte_offset (slave),
+					     &line_offset, html_text_get_font_style (text), text->face, &width, &asc, &dsc);
 	}
 
 	if (width > max_width && len > 1)
@@ -295,18 +332,19 @@ debug_print (HTMLFitType rv, gchar *text, gint len)
 #endif
 
 gint
-html_text_slave_get_line_offset (HTMLTextSlave *slave, gint line_offset, gint offset, HTMLPainter *p)
+html_text_slave_get_line_offset (HTMLTextSlave *slave, gint offset, HTMLPainter *p)
 {
 	HTMLObject *head = HTML_OBJECT (slave->owner)->next;
 
 	g_assert (HTML_IS_TEXT_SLAVE (head));
 
-	if (!html_clueflow_tabs (HTML_CLUEFLOW (HTML_OBJECT (slave)->parent), p) || line_offset == -1)
+	if (!html_clueflow_tabs (HTML_CLUEFLOW (HTML_OBJECT (slave)->parent), p))
 		return -1;
 
 	if (head->y + head->descent - 1 < HTML_OBJECT (slave)->y - HTML_OBJECT (slave)->ascent) {
 		HTMLObject *prev;
 		HTMLTextSlave *bol;
+		gint line_offset = 0;
 
 		prev = html_object_prev (HTML_OBJECT (slave)->parent, HTML_OBJECT (slave));
 		while (prev->y + prev->descent - 1 >= HTML_OBJECT (slave)->y - HTML_OBJECT (slave)->ascent)
@@ -314,9 +352,11 @@ html_text_slave_get_line_offset (HTMLTextSlave *slave, gint line_offset, gint of
 
 		bol = HTML_TEXT_SLAVE (prev->next);
 		return html_text_text_line_length (html_text_slave_get_text (bol),
-						   0, offset - bol->posStart);
+						   &line_offset, slave->posStart + offset - bol->posStart, NULL);
 	} else {
-		html_text_text_line_length (slave->owner->text, &line_offset, offset);
+		gint line_offset = html_text_get_line_offset (slave->owner, p);
+		/* printf ("owner lo %d\n", line_offset); */
+		html_text_text_line_length (slave->owner->text, &line_offset, slave->posStart + offset, NULL);
 		return line_offset;
 	}
 }
@@ -356,14 +396,14 @@ could_remove_leading_space (HTMLTextSlave *slave, gboolean lineBegin)
 	return o->prev ? FALSE : TRUE;
 }
 
-inline gint
+gint
 html_text_slave_nb_width (HTMLTextSlave *slave, HTMLPainter *painter, gint words)
 {
-	return get_words_width (slave->owner, painter, slave->start_word, words)
+	return get_words_width (slave, painter, words)
 		+ (slave->start_word + words == slave->owner->words ? get_next_nb_width (slave, painter) : 0);
 }
 
-inline gchar *
+gchar *
 html_text_slave_remove_leading_space (HTMLTextSlave *slave, HTMLPainter *painter, gboolean lineBegin)
 {
 	gchar *begin;
@@ -400,21 +440,30 @@ hts_fit_line (HTMLObject *o, HTMLPainter *painter,
 	HTMLFitType rv = HTML_FIT_PARTIAL;
 	HTMLTextSlave *slave;
 	HTMLText *text;
+	HTMLObject *prev;
 	guint  pos = 0;
 	gchar *sep = NULL, *lsep;
 	gchar *begin;
 	guint words = 0;
+	gint orig_start_word;
+	gboolean forceFit;
 
 	slave = HTML_TEXT_SLAVE (o);
 	text  = HTML_TEXT (slave->owner);
+	orig_start_word = slave->start_word;
 
 	begin = html_text_slave_remove_leading_space (slave, painter, lineBegin);
 
 	/* printf ("fit_line %d left: %d lspacetext: \"%s\"\n", firstRun, widthLeft, begin); */
 
+	prev = html_object_prev_not_slave (HTML_OBJECT (text));
+	forceFit = orig_start_word == slave->start_word
+		&& prev && html_object_is_text (prev) && HTML_TEXT (prev)->text_len && HTML_TEXT (prev)->text [strlen (HTML_TEXT (prev)->text) - 1] != ' ';
+
 	sep = begin;
-	while (sep && *sep
-	       && widthLeft >= html_text_slave_nb_width (slave, painter, words + 1)) {
+	while ((sep && *sep
+		&& widthLeft >= html_text_slave_nb_width (slave, painter, words + 1))
+	       || (words == 0 && text->words - slave->start_word > 0 && forceFit)) {
 		words ++;
 		lsep   = sep;
 		sep    = strchr (lsep + (words > 1 ? 1 : 0), ' ');
@@ -425,7 +474,7 @@ hts_fit_line (HTMLObject *o, HTMLPainter *painter,
 
 	if (words + slave->start_word == text->words)
 		rv = HTML_FIT_COMPLETE;
-	else if (words == 0 || get_words_width (text, painter, slave->start_word, words) == 0) {
+	else if (words == 0 || get_words_width (slave, painter, words) == 0) {
 		if (!firstRun) {
 			if (slave->posStart == 0 && text->text [0] != ' ' && HTML_OBJECT (text)->prev) {
 				HTMLObject *prev = HTML_OBJECT (text)->prev;
@@ -452,7 +501,7 @@ hts_fit_line (HTMLObject *o, HTMLPainter *painter,
 	if (rv == HTML_FIT_PARTIAL)
 		if (pos < slave->posLen) {
 			split (slave, pos, slave->start_word + words, sep);
-		o->width = get_words_width (text, painter, slave->start_word, words);
+		o->width = get_words_width (slave, painter, words);
 	}
 
 #ifdef HTML_TEXT_SLAVE_DEBUG
@@ -496,7 +545,7 @@ get_ys (HTMLText *text, HTMLPainter *p)
 }
 
 static void
-draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty, gint line_offset)
+draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty)
 {
 	GList *cur = HTML_TEXT (slave->owner)->spell_errors;
 	HTMLObject *obj = HTML_OBJECT (slave);
@@ -504,6 +553,7 @@ draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty, gint
 	guint ma, mi;
 	gint x_off = 0;
 	gint last_off = 0;
+	gint line_offset = html_text_slave_get_line_offset (slave, 0, p);
 	gchar *text = html_text_slave_get_text (slave);
 
 	while (cur) {
@@ -521,21 +571,23 @@ draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty, gint
 			/* printf ("spell error: %s\n", html_text_get_text (slave->owner, off)); */
 			lo = line_offset;
 			
-			glyphs = get_glyphs_part (slave, p, last_off, off - last_off, line_offset, &items);
+			glyphs = get_glyphs_part (slave, p, last_off, off - last_off, &items);
 			html_painter_calc_text_size (p, text,
-						     off - last_off, items, glyphs, &line_offset,
+						     off - last_off, items, glyphs, text - slave->owner->text,
+						     &line_offset,
 						     p->font_style,
 						     p->font_face, &width, &asc, &dsc);
 			glyphs_destroy (glyphs);
 			x_off += width;
 			text = g_utf8_offset_to_pointer (text, off - last_off);
-			glyphs = get_glyphs_part (slave, p, off, len, line_offset, &items);
+			glyphs = get_glyphs_part (slave, p, off, len, &items);
 			x_off += html_painter_draw_spell_error (p, obj->x + tx + x_off,
-								obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-								text, len, items, glyphs);
+								obj->y + ty + get_ys (slave->owner, p),
+								text, len, items, glyphs, text - slave->owner->text);
 			glyphs_destroy (glyphs);
 			last_off = off + len;
-			line_offset += len;
+			if (line_offset != -1)
+				line_offset += len;
 			text = g_utf8_offset_to_pointer (text, len);
 		}
 		if (se->off > slave->posStart + slave->posLen)
@@ -565,53 +617,70 @@ get_items (HTMLTextSlave *slave, HTMLPainter *painter)
 	return slave->items;
 }
 
+static inline GList *
+get_glyphs_base_text (GList *glyphs, PangoItem *item, const gchar *text, gint bytes)
+{
+	PangoGlyphString *str;
+
+	str = pango_glyph_string_new ();
+	pango_shape (text, bytes, &item->analysis, str);
+	glyphs = g_list_prepend (glyphs, str);
+
+	return glyphs;
+}
+
+GList *
+html_get_glyphs_non_tab (GList *glyphs, PangoItem *item, const gchar *text, gint bytes, gint len)
+{
+	gchar *tab;
+
+	while ((tab = memchr (text, (unsigned char) '\t', bytes))) {
+		gint c_bytes = tab - text;
+		if (c_bytes > 0)
+			glyphs = get_glyphs_base_text (glyphs, item, text, c_bytes);
+		text += c_bytes + 1;
+		bytes -= c_bytes + 1;
+	}
+
+	if (bytes > 0)
+		glyphs = get_glyphs_base_text (glyphs, item, text, bytes);
+
+	return glyphs;
+}
+
 static GList *
-get_glyphs_part (HTMLTextSlave *slave, HTMLPainter *painter, guint offset, guint len, gint line_offset, GList **items)
+get_glyphs_part (HTMLTextSlave *slave, HTMLPainter *painter, guint offset, guint len, GList **items)
 {
 	GList *glyphs = NULL;
-	const gchar *text;
 
 	*items = get_items (slave, painter);
-	if (items) {
-		PangoGlyphString *str;
+	if (*items) {
 		PangoItem *item;
-		GList *il;
-		gint bytes, index, current_len, index_in_item;
-		const gchar *slave_text, *item_text;
-		gchar *translated_text, *heap;
+		GList *il = *items;
+		gint index, c_len;
+		gint byte_offset;
+		const gchar *text, *owner_text;
+		gchar *end;
 
-		/* printf ("get_glyphs_part %d,%d\n", offset, len); */
+		owner_text = slave->owner->text;
+		text = g_utf8_offset_to_pointer (html_text_slave_get_text (slave), offset);
+		byte_offset = text - owner_text;
 
-		slave_text = html_text_slave_get_text (slave);
-		text = g_utf8_offset_to_pointer (slave_text, offset);
-		index = offset;
-		while (text - slave->owner->text >= ((PangoItem *) (*items)->data)->offset + ((PangoItem *) (*items)->data)->length) {
-			*items = (*items)->next;
-			/* printf ("skip item, new num_chars: %d\n", ((PangoItem *) (*items)->data)->num_chars); */
+		if (offset) {
+			while (il && (item = (PangoItem *) il->data) && item->offset + item->length <= byte_offset)
+				il = il->next;
+			*items = il;
 		}
-		il = *items;
-		while (il && index - offset < len) {
+
+		index = 0;
+		while (il && index < len) {
 			item = (PangoItem *) il->data;
-			str = pango_glyph_string_new ();
-			item_text = slave->owner->text + item->offset;
-			index_in_item = g_utf8_pointer_to_offset (item_text, text);
-			current_len = MIN (item->num_chars - index_in_item, len - (index - offset));
-			/* printf ("iii: %d oii: %d\n", index_in_item, text - item_text);
-			   printf ("cl: %d offset: %d idx: %d\n", current_len, offset, index); */
+			c_len = MIN (item->num_chars - g_utf8_pointer_to_offset (owner_text + item->offset, text), len - index);
 
-			bytes = g_utf8_offset_to_pointer (text, current_len) - text;
-			heap = NULL;
-			if (bytes > HTML_ALLOCA_MAX)
-				heap = translated_text = g_malloc (bytes);
-			else 
-				translated_text = alloca (bytes);
-			html_replace_tabs (text, translated_text, bytes);
-			pango_shape (translated_text, bytes, &item->analysis, str);
-			g_free (heap);
-
-			glyphs = g_list_prepend (glyphs, str);
-			text = g_utf8_offset_to_pointer (text, current_len);
-			index += current_len;
+			end = g_utf8_offset_to_pointer (text, c_len);
+			glyphs = html_get_glyphs_non_tab (glyphs, item, text, end - text, c_len);
+			text = end;
+			index += c_len;
 			il = il->next;
 		}
 		glyphs = g_list_reverse (glyphs);
@@ -621,44 +690,13 @@ get_glyphs_part (HTMLTextSlave *slave, HTMLPainter *painter, guint offset, guint
 }
 
 static GList *
-get_glyphs (HTMLTextSlave *slave, HTMLPainter *painter, gint line_offset)
+get_glyphs (HTMLTextSlave *slave, HTMLPainter *painter)
 {
 	if (!slave->glyphs) {
-		GList *items = get_items (slave, painter);
-		const gchar *text;
-
-		if (items) {
-			PangoGlyphString *str;
-			PangoItem *item;
-			GList *il = items;
-			gint bytes, index, len;
-			gchar *translated_text, *heap;
-
-			text = html_text_slave_get_text (slave);
-			index = 0;
-			while (il && index < slave->posLen) {
-				item = (PangoItem *) il->data;
-				str = pango_glyph_string_new ();
-				len = MIN (item->num_chars, slave->posLen - index);
-
-				bytes = g_utf8_offset_to_pointer (text, len) - text;
-				heap = NULL;
-				if (bytes > HTML_ALLOCA_MAX)
-					heap = translated_text = g_malloc (bytes);
-				else 
-					translated_text = alloca (bytes);
-				html_replace_tabs (text, translated_text, bytes);
-				pango_shape (translated_text, bytes, &item->analysis, str);
-				g_free (heap);
-
-				slave->glyphs = g_list_prepend (slave->glyphs, str);
-				text = g_utf8_offset_to_pointer (text, len);
-				index += len;
-				il = il->next;
-			}
-			slave->glyphs = g_list_reverse (slave->glyphs);
-		}
+		GList *items;
+		slave->glyphs = get_glyphs_part (slave, painter, 0, slave->posLen, &items);
 	}
+
 	return slave->glyphs;
 }
 
@@ -668,12 +706,11 @@ draw_normal (HTMLTextSlave *self,
 	     GtkHTMLFontStyle font_style,
 	     gint x, gint y,
 	     gint width, gint height,
-	     gint tx, gint ty, gint line_offset)
+	     gint tx, gint ty)
 {
 	HTMLObject *obj;
 	HTMLText *text = self->owner;
 	gchar *str;
-	gint lo;
 
 	obj = HTML_OBJECT (self);
 
@@ -685,17 +722,16 @@ draw_normal (HTMLTextSlave *self,
 		html_painter_set_font_face  (p, HTML_TEXT (self->owner)->face);
 		html_color_alloc (HTML_TEXT (self->owner)->color, p);
 		html_painter_set_pen (p, &HTML_TEXT (self->owner)->color->color);
-		lo = html_text_slave_get_line_offset (self, line_offset, self->posStart, p);
 
 		if (self->posStart > 0)
-			glyphs = get_glyphs_part (self, p, 0, self->posLen, lo, &items);
+			glyphs = get_glyphs_part (self, p, 0, self->posLen, &items);
 		else {
 			items = get_items (self, p);
-			glyphs = get_glyphs (self, p, lo);
+			glyphs = get_glyphs (self, p);
 		}
 
 		html_painter_draw_text (p, obj->x + tx, obj->y + ty + get_ys (text, p),
-					str, self->posLen, items, glyphs, lo);
+					str, self->posLen, items, glyphs, str - self->owner->text, html_text_slave_get_line_offset (self, 0, p));
 
 		if (self->posStart > 0)
 			glyphs_destroy (glyphs);
@@ -708,7 +744,7 @@ draw_highlighted (HTMLTextSlave *slave,
 		  GtkHTMLFontStyle font_style,
 		  gint x, gint y,
 		  gint width, gint height,
-		  gint tx, gint ty, gint line_offset)
+		  gint tx, gint ty)
 {
 	HTMLText *owner;
 	HTMLObject *obj;
@@ -736,25 +772,27 @@ draw_highlighted (HTMLTextSlave *slave,
 	slave_begin = html_text_slave_get_text (slave);
 	highlight_begin = g_utf8_offset_to_pointer (slave_begin, start - slave->posStart);
 
-	lo_start = lo = html_text_slave_get_line_offset (slave, line_offset, slave->posStart, p);
+	lo_start = lo = html_text_slave_get_line_offset (slave, 0, p);
 
 	if (start > slave->posStart) {
-		glyphs1 = get_glyphs_part (slave, p, 0, start - slave->posStart, lo_start, &items1);
-		html_painter_calc_text_size (p, slave_begin, start - slave->posStart, items1, glyphs1, &lo,
+		glyphs1 = get_glyphs_part (slave, p, 0, start - slave->posStart, &items1);
+		html_painter_calc_text_size (p, slave_begin, start - slave->posStart, items1, glyphs1, slave_begin - text, &lo,
 					     font_style, HTML_TEXT (owner)->face, &offset_width, &asc, &dsc);
 	} else
 		offset_width = 0;
 	lo_sel = lo;
 
 	if (len) {
-		glyphs2 = get_glyphs_part (slave, p, start - slave->posStart, len, lo_sel, &items2);
+		glyphs2 = get_glyphs_part (slave, p, start - slave->posStart, len, &items2);
 
-		html_painter_calc_text_size (p, highlight_begin, len, items2, glyphs2, &lo,
+		html_painter_calc_text_size (p, highlight_begin, len, items2, glyphs2, highlight_begin - text, &lo,
 					     font_style, HTML_TEXT (owner)->face, &text_width, &asc, &dsc);
 		/* printf ("s: %d l: %d - %d %d\n", start, len, offset_width, text_width); */
 	} else
 		text_width = 0;
 
+	/* printf ("lo %d - %d(%d) - %d(%d)\n", lo_start, lo_sel, start, lo, end); */
+
 	html_painter_set_font_style (p, font_style);
 	html_painter_set_font_face  (p, HTML_TEXT (owner)->face);
 
@@ -771,7 +809,7 @@ draw_highlighted (HTMLTextSlave *slave,
 	if (len) {
 		html_painter_draw_text (p, obj->x + tx + offset_width, 
 					obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-					highlight_begin, len, items2, glyphs2,
+					highlight_begin, len, items2, glyphs2, highlight_begin - slave->owner->text,
 					lo_sel);
 		if (glyphs2)
 			glyphs_destroy (glyphs2);
@@ -784,7 +822,7 @@ draw_highlighted (HTMLTextSlave *slave,
 		html_painter_draw_text (p,
 					obj->x + tx, obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
 					slave_begin,
-					start - slave->posStart, items1, glyphs1,
+					start - slave->posStart, items1, glyphs1, slave_begin - slave->owner->text,
 					lo_start);
 		if (glyphs1)
 			glyphs_destroy (glyphs1);
@@ -792,12 +830,14 @@ draw_highlighted (HTMLTextSlave *slave,
 
 	/* 2. Draw the rightmost non-highlighted part, if any.  */
 	if (end < slave->posStart + slave->posLen) {
-		glyphs3 = get_glyphs_part (slave, p, start + len - slave->posStart, slave->posLen - start - len + slave->posStart, lo, &items3);
+		gchar *end_text;
+		glyphs3 = get_glyphs_part (slave, p, start + len - slave->posStart, slave->posLen - start - len + slave->posStart, &items3);
+		end_text = g_utf8_offset_to_pointer (highlight_begin, end - start);
 		html_painter_draw_text (p,
 					obj->x + tx + offset_width + text_width,
 					obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
-					g_utf8_offset_to_pointer (highlight_begin, end - start),
-					slave->posStart + slave->posLen - end, items3, glyphs3, lo);
+					end_text,
+					slave->posStart + slave->posLen - end, items3, glyphs3, end_text - slave->owner->text, lo);
 		if (glyphs3)
 			glyphs_destroy (glyphs3);
 	}
@@ -816,7 +856,6 @@ draw (HTMLObject *o,
 	GtkHTMLFontStyle font_style;
 	guint end;
 	GdkRectangle paint;
-	gint line_offset;
 
 	/* printf ("slave draw %p\n", o); */
 
@@ -827,18 +866,17 @@ draw (HTMLObject *o,
 	owner = textslave->owner;
 	ownertext = HTML_TEXT (owner);
 	font_style = html_text_get_font_style (ownertext);
-	line_offset = html_text_get_line_offset (owner, p);
 
 	end = textslave->posStart + textslave->posLen;
 	if (owner->select_start + owner->select_length <= textslave->posStart
 	    || owner->select_start >= end) {
-		draw_normal (textslave, p, font_style, x, y, width, height, tx, ty, line_offset);
+		draw_normal (textslave, p, font_style, x, y, width, height, tx, ty);
 	} else {
-		draw_highlighted (textslave, p, font_style, x, y, width, height, tx, ty, line_offset);
+		draw_highlighted (textslave, p, font_style, x, y, width, height, tx, ty);
 	}
 
 	if (HTML_TEXT (textslave->owner)->spell_errors)
-		draw_spell_errors (textslave, p, tx ,ty, line_offset);
+		draw_spell_errors (textslave, p, tx ,ty);
 }
 
 static gint
@@ -871,8 +909,7 @@ get_offset_for_pointer (HTMLTextSlave *slave, HTMLPainter *painter, gint x, gint
 	GtkHTMLFontStyle font_style;
 	guint width, prev_width;
 	guint i;
-	gint line_offset, lo, word, ww;
-	gchar *space;
+	gint line_offset, lo, word;
 	gchar *text;
 
 	g_return_val_if_fail (slave != NULL, 0);
@@ -883,12 +920,12 @@ get_offset_for_pointer (HTMLTextSlave *slave, HTMLPainter *painter, gint x, gint
 	x -= HTML_OBJECT (slave)->x;
 
 	prev_width  = 0;
-	line_offset = html_text_slave_get_line_offset (slave, html_text_get_line_offset (slave->owner, painter), 0, painter);
+	line_offset = html_text_slave_get_line_offset (slave, 0, painter);
 
 	i = 1;
 	word = 1;
 	text = html_text_slave_get_text (slave);
-	space = strchr (text, ' ');
+	/*space = strchr (text, ' ');
 	if (space && g_utf8_pointer_to_offset (text, space) <= slave->posLen) {
 		html_text_request_word_width (owner, painter);
 
@@ -902,7 +939,7 @@ get_offset_for_pointer (HTMLTextSlave *slave, HTMLPainter *painter, gint x, gint
 			word ++;
 			ww = get_words_width (owner, painter, slave->start_word, word);
 		}
-	}
+		}*/
 
 	for ( ; i <= slave->posLen; i++) {
 		GList *items;
@@ -910,8 +947,9 @@ get_offset_for_pointer (HTMLTextSlave *slave, HTMLPainter *painter, gint x, gint
 		gint asc, dsc;
 
 		lo = line_offset;
-		glyphs = get_glyphs_part (slave, painter, 0, i, lo, &items);
-		html_painter_calc_text_size (painter, text, i, items, glyphs, &lo, font_style, owner->face, &width, &asc, &dsc);
+		glyphs = get_glyphs_part (slave, painter, 0, i, &items);
+		html_painter_calc_text_size (painter, text, i, items, glyphs, html_text_slave_get_start_byte_offset (slave), &lo, font_style, owner->face, &width, &asc, &dsc);
+		glyphs_destroy (glyphs);
 
 		if ((width + prev_width) / 2 >= x)
 			return i - 1;
